EVENT_QUERY = '''
query TournamentEvents($slug: String!) {
  tournament(slug: $slug) {
    events {
      id
      slug
      name
      numEntrants
      videogame {
        id
      }
      teamRosterSize {
        maxPlayers
        minPlayers
      }
      tournament {
        name
        id
      }
    }
  }
}
'''

RESULTS_QUERY = '''
query EventSets($slug: String!, $page: Int!) {
  event(slug: $slug) {
    slug
    tournament {
      id
      name
    }
    id
    name
    sets(page: $page, perPage: 18, sortType: STANDARD) {
      nodes {
        fullRoundText
        id
        slots {
          standing {
            placement
            stats {
              score {
                value
              }
            }
          }
          entrant {
            id
            name
            participants {
              entrants {
                id
              }
              player {
                id
                gamerTag
                user {
                  slug
                }
              }
            }
          }
        }
      }
    }
  }
}
'''

PLAYERS_QUERY = '''
query ($slug: String!) {
  user(slug: $slug) {
    player {
      id
      gamerTag
    }
    name
    location {
      country
      state
      city
    }
  }
}
'''


import gspread
import pandas as pd
import csv
import time

green = {
    "red": 0.0,
    "green": 1.0,
    "blue": 0.0
}
red = {
    "red": 1.0,
    "green": 0.0,
    "blue": 0.0
}
yellow = {
    "red": 1.0,
    "green": 1.0,
    "blue": 0.0
}

def importSpreadsheet(key_location: str, spreadsheet_name: str, worksheet_name: str, csv_location: str):
    gc = gspread.service_account(key_location) # connect to google account using api key
    sh = gc.open(spreadsheet_name) #connect to spreadsheet. defaults to first worksheet
    worksheet = sh.worksheet(worksheet_name) #open worksheet to write to
    worksheet.clear() #clear worksheet and dump csv
    sh.values_update(
        worksheet_name,
        params={'valueInputOption': 'USER_ENTERED'},
        body={'values': list(csv.reader(open(csv_location)))}
    )
    # Color cells
    size = pd.read_csv(csv_location).shape[0] # use pandas to get how many players there are in csv
    for row in range(2, 2+size): # Start on row 2
        for column in range(2,2+size): # start on second column
            val = worksheet.cell(row, column).value
            if val != 'N/A':
                colorCell(worksheet, chr(column+64) + str(row), val.split('-'))
                time.sleep(60/100) # google api only allows 100 writes a minute
  

def colorCell(worksheet, cell, score):
    #win
    if int(score[0]) > int(score[1]):
        worksheet.format(cell, {
                    "backgroundColor": green,
                    "horizontalAlignment": "CENTER",
        })
    #loss
    elif int(score[0]) < int(score[1]):
        worksheet.format(cell, {
                    "backgroundColor": red,
                    "horizontalAlignment": "CENTER",
        })
    #tie
    else:
        worksheet.format(cell, {
                "backgroundColor": yellow,
                "horizontalAlignment": "CENTER",
        })


from H2H_Creator_startgg import events, results, players, creator

class H2HMaker(object):
    def __init__(self, key, save_json:bool, sleep_time=15): # Initializes object
        self.key = key
        self.header = {"Authorization": "Bearer " + key}
        self.save_json = save_json
        self.sleep_time = sleep_time # Sleep time is how long it waits between 6 queries to sleep

    def set_key(self, new_key): # Sets new key
        self.key = new_key
        self.header = {"Authorization": "Bearer " + new_key}

    def set_save_json(self, new_save_json):
        self.save_json = new_save_json

    def set_sleep_time(self, new_sleep_time):
        self.sleep_time = new_sleep_time

    def print_key(self):
        print(self.key)
    
    def print_header(self):
        print(self.header)

    def print_sleep_time(self):
        print(self.sleep_time)

    def get_players_info(self, player_list:list): # List of slugs
        return players.get_players_info(player_list, self.save_json, self.header, self.sleep_time)
        
    def get_events(self, tournaments:list, game):
        return events.get_events(tournaments, game, self.save_json, self.header, self.sleep_time)

    def get_results(self, events:list, players:list): 
        return results.get_results(events, players, self.save_json, self.header, self.sleep_time)

    def create_h2h_spreadsheet(self, events:list, players:list):
        return creator.h2h_spreadsheet(events, players, self.save_json, self.header, self.sleep_time)

    def create_win_loss_spreadsheet(self, events:list, players:list):
        return creator.wl_spreadsheet(events, players, self.save_json, self.header, self.sleep_time)



import json
import csv
from H2H_Creator_startgg.players import get_players_info
from H2H_Creator_startgg.results import get_results

def create_player_dictionary(results:list, players:list, save_json:bool, header, sleep_time):
    players_info = get_players_info(players, save_json, header, sleep_time)

    player_dict = {}
    for p in players_info: # Initializes dictionary
        player_dict[players_info[p]["player"]["gamerTag"]] = {"Slug": p, "ID": players_info[p]["player"]["id"], 'Events': [], 'Sets': [], 'W': [], 'L': []}

    for tournament in results:
        for set in results[tournament]['sets']:
            event_slug = tournament
            entrant1 = set['slots'][0]['entrant']['participants'][0]['player']['gamerTag'].split(' | ')[-1].strip() # Gamertag
            entrant1_slug = set['slots'][0]['entrant']['participants'][0]['player']['user']['slug'].split('/')[1] # Slug
            entrant1_score = set['slots'][0]['standing']['stats']['score']['value'] # Score
            entrant2 = set['slots'][1]['entrant']['participants'][0]['player']['gamerTag'].split(' | ')[-1].strip() # Gamertag
            entrant2_slug = set['slots'][1]['entrant']['participants'][0]['player']['user']['slug'].split('/')[1] # Slug
            entrant2_score = set['slots'][1]['standing']['stats']['score']['value'] # Score

            if set['slots'][0]['standing'] is not None: # If match is completed
                # Both players in the set are important
                if (entrant1_slug in players and entrant2_slug in players): 
                    if (event_slug not in player_dict[entrant1]['Events']): # Adds event to event list
                        player_dict[entrant1]['Events'].append(event_slug)
                    if (event_slug not in player_dict[entrant2]['Events']):# Adds event to event list
                        player_dict[entrant2]['Events'].append(event_slug)

                    if (set not in player_dict[entrant1]['Sets']): # Prevents duplicate sets, happens with shitty apis lmao
                        player_dict[entrant1]['Sets'].append(set)
                        player_dict[entrant2]['Sets'].append(set)

                        # Adding wins and losses
                        if (entrant1_score > entrant2_score):
                            player_dict[entrant1]['W'].append(entrant2)
                            player_dict[entrant2]['L'].append(entrant1)
                        elif (entrant2_score > entrant1_score):
                            player_dict[entrant2]['W'].append(entrant1)
                            player_dict[entrant1]['L'].append(entrant2)

                # Player 1 in the set is important
                elif (entrant1_slug in players): 
                    # Checking for event attendance marked already or not
                    if (event_slug not in player_dict[entrant1]['Events']):
                        player_dict[entrant1]['Events'].append(event_slug)

                    # Add set to sets
                    if (set not in player_dict[entrant1]['Sets']): # Prevents duplicate sets, happens with shitty apis lmao
                        player_dict[entrant1]['Sets'].append(set)

                    # If known player lost
                    if (entrant2_score > entrant1_score):
                        player_dict[entrant1]['L'].append(entrant2)

                # Player 2 in the set is important
                elif (entrant2_slug in players): 
                    # Checking for event attendance marked already or not
                    if (event_slug not in player_dict[entrant2]['Events']):
                        player_dict[entrant2]['Events'].append(event_slug)

                    # Add set to sets
                    if (set not in player_dict[entrant2]['Sets']): # Prevents duplicate sets, happens with shitty apis lmao
                        player_dict[entrant2]['Sets'].append(set)

                    # If known player lost
                    if (entrant1_score > entrant2_score):
                        player_dict[entrant2]['L'].append(entrant1)

    if save_json: # Outputting json file if flag activated
        with open('players_results.json', 'w+', encoding='utf-8') as outfile:
            json.dump(player_dict, outfile, indent=4)

    return player_dict

def to_csv_h2h(data, output='H2H_table.csv'):
    players = []
    for p in data:
        players.append(p)

    i = 0
    with open(output, 'w', encoding='utf-8') as new_file:
        # Top left corner
        new_file.write('ETossed,')

        # Header row
        for p in players:
            new_file.write(p + ',')

        # Other losses and newline after header        
        new_file.write('Other losses\n')
        
        # Setup rows
        for p in players:
            new_file.write(p + ',')
            for j in range(len(players)):
                if (i == j):
                    new_file.write('N/A,')
                else:
                    new_file.write('0-0,')

            # Increment i and write other losses and newline
            new_file.write(',\n')
            i += 1

    # Open csv file
    csv_file = csv.reader(open(output, encoding='utf-8'))
    csv_lines = list(csv_file)

    # For each w/l find index in player_list
    for i in range(len(players)):
        cur_player = data[players[i]]
        print(players[i])
        wins = cur_player['W']
        losses = cur_player['L']
        for op in wins:
            # Access cell through csv built in python functions
            j = players.index(op)
            score = csv_lines[i+1][j+1]
            print("i+1: " + str(i+1) + ", j+1: " + str(j+1))

            # Split cell by '-' character and modify score
            temp_score = score.split('-')
            temp_score[0] = str(int(temp_score[0]) + 1)
            new_score = "-".join(temp_score)
            csv_lines[i+1][j+1] = new_score

        other_losses = []

        for op in losses:
            # Access cell through csv built in python functions
            if op in players:
                j = players.index(op)
                score = csv_lines[i+1][j+1]
                print("i+1: " + str(i+1) + ", j+1: " + str(j+1))

                # Split cell by '-' character and modify score
                temp_score = score.split('-')
                temp_score[1] = str(int(temp_score[1]) + 1)
                new_score = "-".join(temp_score)
                csv_lines[i+1][j+1] = new_score
            else: # For non player list losses
                other_losses.append(op)
        
        if other_losses != []:
            csv_lines[i+1][len(players)+1] = ", ".join(other_losses)

    # Write csv
    writer = csv.writer(open(output,'w',encoding='utf-8',newline=''))
    writer.writerows(csv_lines)

def to_csv_wl(data, output='WL_table.csv'):
    players = []
    for p in data:
        players.append(p)

    with open(output, 'w', encoding='utf-8') as new_file:
        # Top left corner
        new_file.write('ETossed,')

        # Header row
        new_file.write('Wins,Losses\n')

        for player in players:
            # Initialization
            wins = {}
            losses = {}

            new_file.write(player + ',\"') # Initial quote
            for op in data[player]['W']:
                if op in wins:
                    wins[op] += 1
                else:
                    wins[op] = 1
            
            # Adding wins to column
            for w in sorted(wins, key=str.casefold):
                if wins[w] == 1:
                    new_file.write(w + ", ")
                else:
                    new_file.write(w + "(x" + str(wins[w]) + "), ")

            # Closing quote and new quote
            new_file.write('\",\"')

            for op in data[player]['L']:
                # If already in lossses
                if op in losses:
                    losses[op] += 1
                else:
                    losses[op] = 1
            
            # Adding wins to column
            for l in sorted(losses, key=str.casefold):
                if losses[l] == 1:
                    new_file.write(l + ", ")
                else:
                    new_file.write(l + "(x" + str(losses[l]) + "), ")

            # Closing quote
            new_file.write('\",')

            # Print newline
            new_file.write('\n')

def h2h_spreadsheet(events:list, players:list, save_json:bool, header, sleep_time):
    results = get_results(events, players, save_json, header, sleep_time) # Gets results
    player_data = create_player_dictionary(results, players, save_json, header, sleep_time) # Gets player data

    to_csv_h2h(player_data)

def wl_spreadsheet(events:list, players:list, save_json:bool, header, sleep_time):
    results = get_results(events, players, save_json, header, sleep_time) # Gets results
    player_data = create_player_dictionary(results, players, save_json, header, sleep_time) # Gets player data

    to_csv_wl(player_data)



# File for getting all results from list of players and events

import json
from time import sleep
from H2H_Creator_startgg.api import run_query
from H2H_Creator_startgg.events import get_events
from H2H_Creator_startgg.players import get_players_info
from H2H_Creator_startgg.queries import RESULTS_QUERY
from H2H_Creator_startgg.exceptions import *

def get_results(events:list, players:list, save_json:bool, header, sleep_time):
    results = {}

    for e in events:
        print("Now doing event {}".format(e))
        sets = [] # List of sets
        i = 1 # Page

        done = False

        while (not done):
            if i % 35 == 0: # Sleeping so startgg server doesn't hate me
                print("Sleeping for {} seconds".format(sleep_time))
                sleep(sleep_time)

            variables = {"slug": e, "page": i}
            response = run_query(RESULTS_QUERY, variables, header) # Send request
            print("Page {}".format(i)) # Console logging

            if response == 500: # If random server error
                print("Retrying in 15 seconds")
                i -= 1
                sleep(15)

            # ERROR CHECKING
            if response == 500: # Server error
                print("You got a server error, retrying in 15 seconds")
            elif 'data' not in response: # Error
                return
            elif response['data']['event'] is None: # Error
                return
            elif response['data']['event']['sets']['nodes'] is None: # Error
                return
            elif response['data']['event']['sets']['nodes'] == []: # If pagination completed
                done = True
            else:
                i += 1 # iteration for next time

            for s in response['data']['event']['sets']['nodes']: # Iterate through all sets
                player1 = s['slots'][0]['entrant']['participants'][0]['player']['user']['slug'].split('/')[1] # Gets user slug of player #1
                player2 = s['slots'][1]['entrant']['participants'][0]['player']['user']['slug'].split('/')[1] # Gets user slug of player #2
                if (player1 in players or player2 in players): # If either player is in the list
                    if (s not in sets): # Prevent duplicate sets
                        sets.append(s) # Append set to sets list

        results[e] = {
            # Will add more info
            # 'tournamentName': e['tournament']['name'],
            # 'tournamentId': e['tournament']['name'],
            # 'eventName': e['name'],
            # 'eventId': e['id'],
            'sets': sets
        }

    if save_json: # Outputting json file if flag activated
        with open('tournament_results.json', 'w+', encoding='utf-8') as outfile:
            json.dump(results, outfile, indent=4)

    return results


# File for getting all events from tournaments certain for a certain game
# TODO: Time stuff

import json
from time import sleep
from H2H_Creator_startgg.api import run_query
from H2H_Creator_startgg.queries import EVENT_QUERY


def get_events(tournaments:list, game:int, save_json:bool, header:dict, sleep_time:int):
    events = []
    substrings = ['singles', '1v1', 'championships', 'ladder']
    bad_substrings = ['volleyball', 'doubles', 'amateur']

    i = 0
    while(i < len(tournaments)):
        t = tournaments[i]
        changed = False # If event exists and array changes
        print("Tournament {}".format(tournaments[i])) # Console logging
        i += 1 # Iterate for next time

        if (i+1) % 35 == 0: # Sleeping so startgg server doesn't hate me
            print("Sleeping for 15 seconds") # Console logging
            sleep(sleep_time)

        variables = {"slug": t}
        response = run_query(EVENT_QUERY, variables, header) # Get response from server

        if response == 500: # If random server error
            print("Retrying in 15 seconds")
            i -= 1
            sleep(15)

        if response['data']['tournament'] is None: # Error checking
            print("ERROR: {} is not a valid tournament slug".format(t))

        if response['data']['tournament']['events'] == []: # Error checking
            print("ERROR: {} has no events".format(t))

        for e in response['data']['tournament']['events']: # Loop to find specific event
            if e['videogame']['id'] == game:
                if e['teamRosterSize'] is None or e['teamRosterSize']['maxPlayers'] == 1:
                    if any(x in e['name'].lower() for x in substrings):
                        if any(y in e['name'].lower() for y in bad_substrings):
                            continue
                        else:
                            del e['teamRosterSize']
                            events.append(e)
                            changed = True
                
        if (not changed): # If no event found
            print("ERROR: {} had no matching singles events for desired videogame ID".format(t))
        
    if save_json: # Outputting json file if flag activated
        with open('events.json', 'w+', encoding='utf-8') as outfile:
            json.dump(events, outfile, indent=4)

    return events


# File for getting all player info from the list of players provided

import json
from time import sleep
from H2H_Creator_startgg.api import run_query
from H2H_Creator_startgg.queries import PLAYERS_QUERY
from H2H_Creator_startgg.exceptions import *

# ["2a371960"]

def get_players_info(players:list, save_json:bool, header, sleep_time):
    players_info = {}

    i = 0
    while (i < len(players)):
        variables = {"slug": players[i]}
        response = run_query(PLAYERS_QUERY, variables, header) # Get response from server
        print("Trying player {}".format(players[i])) # Console logging

        if i % 35 == 0: # Sleeping so startgg server doesn't hate me
            print("Sleeping for {} seconds".format(sleep_time))
            sleep(sleep_time)

        if response == 500: # If random server error
            print("Retrying in 15 seconds")
            i -= 1
            sleep(15)

        if response['data']['user'] is None: # Error Checking
            return
        if response['data']['user']['player'] is None: # Error Checking
            return

        players_info[players[i]] = response['data']['user']

        i += 1 # iteration

    if save_json: # Outputting json file if flag activated
        with open('players.json', 'w+', encoding='utf-8') as outfile:
            json.dump(players_info, outfile, indent=4)

    return players_info



import requests
from H2H_Creator_startgg.exceptions import TooManyRequestsError, ResponseError, RequestError, ServerError, NoIdeaError

def run_query(query, variables, header):
    json_request = {'query': query, 'variables': variables}
    try:
        request = requests.post(url='https://api.start.gg/gql/alpha', json=json_request, headers=header)
        if request.status_code == 400:
            raise RequestError
        elif request.status_code == 429:
            raise TooManyRequestsError
        elif 400 <= request.status_code < 500:
            raise ResponseError
        elif 500 <= request.status_code < 600:
            raise ServerError
        elif 300 <= request.status_code < 400:
            raise NoIdeaError

        response = request.json()
        return response

    except RequestError:
        print("Error 400: Bad request (probably means your key is wrong)")
        return

    except TooManyRequestsError:
        print("Error 429: Sending too many requests right now")
        return

    except ResponseError:
        print("Error {}: Unknown request error".format(request.status_code))
        return

    except ServerError:
        print("Error {}: Unknown server error".format(request.status_code))
        return 500

    except NoIdeaError:
        print("Error {}: I literally have no idea how you got this status code, please send this to me".format(request.status_code))
        return